"""Testing various commands of clifold"""
import os
import pytest

from clifold import clifold_main


def test_as_module():
    exit_status = os.system('python3 -m clifold')
    assert exit_status == 256


def test_help():
    help_status = os.system('clif --help')
    h_status = os.system('clif -h')
    assert help_status == 0
    assert h_status == 0


def test_version():
    ver_status = os.system('clif --version')
    v_status = os.system('clif -V')
    assert ver_status == 0
    assert v_status == 0


def test_git():
    git_status = os.system('clif --git')
    g_status = os.system('clif -g')
    assert git_status == 512
    assert g_status == 512


def test_pkg():
    pkg_status = os.system('clif --pkg')
    p_status = os.system('clif -p')
    assert pkg_status == 512
    assert p_status == 512


def test_init():
    init_status = os.system('clif --init')
    i_status = os.system('clif -i')
    assert init_status == 512
    assert i_status == 512


def test_nogit():
    nogit_status = os.system('clif --no-git')
    ng_status = os.system('clif -ng')
    assert nogit_status == 512
    assert ng_status == 512


def test_nopkg():
    nopkg_status = os.system('clif --no-pkg')
    np_status = os.system('clif -np')
    assert nopkg_status == 512
    assert np_status == 512


def test_noinit():
    noinit_status = os.system('clif --no-init')
    ni_status = os.system('clif -ni')
    assert noinit_status == 512
    assert ni_status == 512


def test_proj():
    np_status = os.system('clif testing -g -np -i')
    ngnp_status = os.system('clif testing -ng -np -i')
    ngnpni_status = os.system('clif testing -ng -np -ni')
    assert np_status == 0
    assert ngnp_status == 0
    assert ngnpni_status == 0


def test_proje():
    np_status = os.system('clif testing -g -np -i')
    ngnp_status = os.system('clif testing -ng -np -i')
    ngnpni_status = os.system('clif testing -ng -np -ni')
    assert np_status == 0
    assert ngnp_status == 0
    assert ngnpni_status == 0


def test_project():
    np_status = os.system('clif testing --g --np --i')
    ngnp_status = os.system('clif testing --ng --np --i')
    ngnpni_status = os.system('clif testing --ng --np --ni')
    assert np_status == 512
    assert ngnp_status == 512
    assert ngnpni_status == 512


def test_arbitary_commands():
    arbitary_status = os.system('clif flaksjdfljasd --g -asd-fasd-f-dgsdf-adsf')
    assert arbitary_status == 512


def test_proj_only():
    output = os.system('clif torch')
    assert output == 256


def test_cli():
    with pytest.raises(SystemExit):
        clifold_main.cli()
        pytest.fail("CLI doesn't asking for a command argument...")



"""Install packages with pip."""
import os
import subprocess
# import shutil
import shlex


def pip(pkg, root_path, pname):
    '''
    Packages installation function.

    Args:
        pkg: True or False
        project_path: absolute path to the project
        pname: project name

    Returns:
        Packages installed or not installed
    '''
    if pkg:
        os.chdir(root_path)
        pip_path = "bin/pip3"
        pkg = input("\n📦 Packages you want to install: ")
        pkgs = shlex.split(pkg)
        args = [pip_path, "install"]
        for i in range(len(args)):
            pkgs.insert(i, args[i])

        print('\n🔰 Downloading packages...\n')
        subprocess.run(pkgs, check=True)
        return 1
    else:
        print("\nNo package downloaded...")
        return 0



"""Checking the output of test_commands.py"""
import os


def test_check():
    os.system("cd ..")
    output1 = os.path.exists("testing-venv")
    output2 = os.path.exists("testing-venv/bin")
    output3 = os.path.exists("testing-venv/include")
    output4 = os.path.exists("testing-venv/lib")
    output5 = os.path.exists("testing-venv/pyvenv.cfg")
    output6 = os.path.exists("testing-venv/testing")
    output7 = os.path.exists("testing-venv/testing/testing")
    output8 = os.path.exists("testing-venv/testing/testing/__init__.py")
    output9 = os.path.exists("testing-venv/testing/testing/main.py")
    output10 = os.path.exists("testing-venv/testing/tests")
    output11 = os.path.exists("testing-venv/testing/tests/__init__.py")
    output12 = os.path.exists("testing-venv/testing/tests/test_main.py")
    output13 = os.path.exists("testing-venv/testing/.gitignore")
    output14 = os.path.exists("testing-venv/testing/MANIFEST.in")
    output15 = os.path.exists("testing-venv/testing/README.md")
    output16 = os.path.exists("testing-venv/testing/setup.py")

    assert output1 == True
    assert output2 == True
    assert output3 == True
    assert output4 == True
    assert output5 == True
    assert output6 == True
    assert output7 == True
    assert output8 == True
    assert output9 == True
    assert output10 == True
    assert output11 == True
    assert output12 == True
    assert output13 == True
    assert output14 == True
    assert output15 == True
    assert output16 == True


def test_torch():
    os.system("cd ..")
    output1 = os.path.exists("torch-venv")
    output2 = os.path.exists("torch-venv/bin")
    output3 = os.path.exists("torch-venv/include")
    output4 = os.path.exists("torch-venv/lib")
    output5 = os.path.exists("torch-venv/pyvenv.cfg")
    output6 = os.path.exists("torch-venv/torch")
    output7 = os.path.exists("torch-venv/torch/torch")
    output8 = os.path.exists("torch-venv/torch/torch/__init__.py")
    output9 = os.path.exists("torch-venv/torch/torch/main.py")
    output10 = os.path.exists("torch-venv/torch/tests")
    output11 = os.path.exists("torch-venv/torch/tests/__init__.py")
    output12 = os.path.exists("torch-venv/torch/tests/test_main.py")
    output13 = os.path.exists("torch-venv/torch/.gitignore")
    output14 = os.path.exists("torch-venv/torch/MANIFEST.in")
    output15 = os.path.exists("torch-venv/torch/README.md")
    output16 = os.path.exists("torch-venv/torch/setup.py")

    assert output1 == True
    assert output2 == True
    assert output3 == True
    assert output4 == True
    assert output5 == True
    assert output6 == True
    assert output7 == True
    assert output8 == True
    assert output9 == True
    assert output10 == True
    assert output11 == True
    assert output12 == True
    assert output13 == False
    assert output14 == False
    assert output15 == False
    assert output16 == False



"""Setup file for clifold python module"""
from setuptools import find_packages, setup
from os import path

from clifold.clifold_main import __version__


__version__ = __version__()
here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

tests_require = [
    "pytest",
    "flake8"
]

extras = {
    "test": tests_require
}

setup(
    name="clifold",
    version=__version__,
    author="Jeff Yang",
    author_email="ydc.jeff@gmail.com",
    description="🚀 A CLI tool for scaffolding any Python Projects 🚀",
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords="scaffold python project cli",
    license="MIT",
    url="https://github.com/ydcjeff/clifold",
    packages=find_packages(),
    entry_points={
        "console_scripts": [
            "clif=clifold.clifold_main:cli"
        ],
    },
    install_requires=[
        "requests"
    ],
    extras_require=extras,
    tests_require=tests_require,
    python_requires=">=3.6.0",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Utilities"
    ],
)



"""Create a project with venv."""
import os
import sys


def create(pname):
    '''
    Project creation function.

    Args:
        pname: project_name

    Returns:
        Absolute Project Path
    '''
    proj_files = ['__init__', 'main']
    test_files = ['__init__', 'test_main']
    dirs_folders = [f'{pname}/{pname}', f'{pname}/tests']

    current_path = os.getcwd()
    exist_path = os.path.join(current_path, f'{pname}-venv')
    project_path = os.path.join(exist_path, pname)
    if os.path.exists(exist_path):
        print("Project {} already exists in {}.\n".format(pname, current_path))
        sys.exit()
    else:
        os.system(f'python3 -m venv {pname}-venv')
        os.chdir(exist_path)
        for i in range(len(dirs_folders)):
            os.makedirs(os.path.join(exist_path, dirs_folders[i]))
            for j in range(len(dirs_folders)):
                if i == 0:
                    open(f"{dirs_folders[i]}/{proj_files[j]}.py", 'x').close()
                else:
                    open(f"{dirs_folders[i]}/{test_files[j]}.py", 'x').close()

        print("\n🎉 Project {} created in {}.\n".format(pname, current_path))

    return project_path, exist_path
